@@ -16,7 +16,7 @@
 # along with QuantTools. If not, see <http://www.gnu.org/licenses/>.
 
 #' Download historical market data
-#' @title Download historical market data
+#'
 #' @param symbol symbol name
 #' @param from,to text dates in format \code{"YYYY-mm-dd"}
 #' @param period candle period \code{tick, 1min, 5min, 10min, 15min, 30min, hour, day, week, month}
@@ -25,8 +25,7 @@
 #' @param code futures or option code name, e.g. \code{"RIU6"}
 #' @param contract,frequency,day_exp same as in \code{\link{gen_futures_codes}}
 #' @name get_market_data
-#' @details
-#' Use external websites to get desired symbol name for
+#' @details Use external websites to get desired symbol name for
 #' \href{https://www.finam.ru/profile/moex-akcii/sberbank/export/}{Finam},
 #' \href{https://www.moex.com/en/derivatives/contracts.aspx}{MOEX},
 #' \href{https://www.iqfeed.net/symbolguide/index.cfm?symbolguide=lookup}{IQFeed},
@@ -343,7 +342,7 @@ get_iqfeed_data = function( symbol, from, to = from, period = 'day', local = FAL
 
   curr_date = format( Sys.Date() )
   if( from > curr_date ) from = to = curr_date
-  if( to   > curr_date ) to = curr_date
+  #if( to   > curr_date ) to = curr_date
 
   if( local ){
 
