@@ -27,6 +27,7 @@
 #include "../CppToR.h"
 #include "../ListBuilder.h"
 #include "../NPeriods.h"
+#include "../Alarm.h"
 #include <map>
 #include <cmath>
 #include <Rcpp.h>
@@ -42,7 +43,7 @@ private:
   std::vector< std::string > TradeSideString  = { "long", "short" };
   std::vector< std::string > OrderTypeString  = { "market", "limit" };
   std::vector< std::string > OrderStateString = { "new", "registered", "executed", "cancelling", "cancelled" };
-  std::vector< std::string > TradeStateString = { "new", "opened", "closed" };
+  std::vector< std::string > TradeStateString = { "new", "opened", "closed", "closing" };
 
   std::vector<Order*> orders;
   std::vector<Order*> ordersProcessed;
@@ -62,6 +63,15 @@ private:
 
   std::string timeZone;
 
+  Alarm alarmMarketOpen;
+  Alarm alarmMarketClose;
+
+
+  double startTradingTime    = 0;
+  double stopTradingDrawdown = NAN;
+  double stopTradingLoss     = NAN;
+  bool   isTradingStopped = false;
+
   void FormCandle( const Tick& tick ) {
 
     bool startOver = candle.time != floor( tick.time / timeFrame ) * timeFrame + timeFrame;
@@ -86,6 +96,8 @@ public:
 
   std::function< void( const Tick&   ) > onTick;
   std::function< void( const Candle& ) > onCandle;
+  std::function< void( ) > onMarketOpen;
+  std::function< void( ) > onMarketClose;
 
   Processor( int timeFrame, double latencySend = 0.001, double latencyReceive = 0.001 ) :
 
@@ -109,14 +121,14 @@ public:
     Rcpp::StringVector names = cost.attr( "names" );
 
     bool hasPointValue = std::find( names.begin(), names.end(), "pointValue" ) != names.end();
-    bool hasCancel	   = std::find( names.begin(), names.end(), "cancel"     ) != names.end();
-    bool hasOrder	     = std::find( names.begin(), names.end(), "order"      ) != names.end();
+    bool hasCancel     = std::find( names.begin(), names.end(), "cancel"     ) != names.end();
+    bool hasOrder      = std::find( names.begin(), names.end(), "order"      ) != names.end();
     bool hasStockAbs   = std::find( names.begin(), names.end(), "stockAbs"   ) != names.end();
-    bool hasTradeAbs	 = std::find( names.begin(), names.end(), "tradeAbs"   ) != names.end();
-    bool hasTradeRel	 = std::find( names.begin(), names.end(), "tradeRel"   ) != names.end();
-    bool hasLongAbs	   = std::find( names.begin(), names.end(), "longAbs"    ) != names.end();
-    bool hasLongRel	   = std::find( names.begin(), names.end(), "longRel"    ) != names.end();
-    bool hasShortAbs	 = std::find( names.begin(), names.end(), "shortAbs"   ) != names.end();
+    bool hasTradeAbs   = std::find( names.begin(), names.end(), "tradeAbs"   ) != names.end();
+    bool hasTradeRel   = std::find( names.begin(), names.end(), "tradeRel"   ) != names.end();
+    bool hasLongAbs    = std::find( names.begin(), names.end(), "longAbs"    ) != names.end();
+    bool hasLongRel    = std::find( names.begin(), names.end(), "longRel"    ) != names.end();
+    bool hasShortAbs   = std::find( names.begin(), names.end(), "shortAbs"   ) != names.end();
     bool hasShortRel   = std::find( names.begin(), names.end(), "shortRel"   ) != names.end();
 
     if( hasPointValue ) this->cost.pointValue = cost["pointValue"];
@@ -132,10 +144,92 @@ public:
 
   }
 
+  void SetStop( Rcpp::List stop ) {
+
+    Rcpp::StringVector names = stop.attr( "names" );
+
+    bool hasDrawDown = std::find( names.begin(), names.end(), "drawdown" ) != names.end();
+    bool hasLoss	   = std::find( names.begin(), names.end(), "loss"     ) != names.end();
+
+    if( hasDrawDown ) stopTradingDrawdown = stop["drawdown" ];
+    if( hasLoss     ) stopTradingLoss     = stop["loss"     ];
+
+
+  }
+  void SetTradingHours( double start, double end ) {
+
+    alarmMarketOpen .Set( start );
+    alarmMarketClose.Set( end   );
+
+  }
+  bool IsTradingHoursSet() { return alarmMarketClose.IsSet() and alarmMarketOpen.IsSet(); }
+  void SetLatencyReceive( double latencyReceive ) {
+
+    this->latencyReceive = latencyReceive;
+
+  }
+  void SetLatencySend( double latencySend ) {
+
+    this->latencySend = latencySend;
+
+  }
+  void SetLatency( double latency ) {
+
+    latencySend    = latency / 2;
+    latencyReceive = latency / 2;
+
+  }
+  void SetStartTradingTime( double startTradingTime ) {
+
+    this->startTradingTime = startTradingTime;
+
+  }
+
+  void SetOptions( Rcpp::List options ) {
+
+    Rcpp::StringVector names = options.attr( "names" );
+
+    bool hasCost           = std::find( names.begin(), names.end(), "cost"            ) != names.end();
+    bool hasStop           = std::find( names.begin(), names.end(), "stop"            ) != names.end();
+    bool hasTradeStart     = std::find( names.begin(), names.end(), "trade_start"     ) != names.end();
+    bool hasLatency        = std::find( names.begin(), names.end(), "latency"         ) != names.end();
+    bool hasLatencyReceive = std::find( names.begin(), names.end(), "latency_receive" ) != names.end();
+    bool hasLatencySend    = std::find( names.begin(), names.end(), "latency_send"    ) != names.end();
+    bool hasTradingHours   = std::find( names.begin(), names.end(), "trading_hours"   ) != names.end();
+
+    if( hasTradingHours ) {
+
+      Rcpp::NumericVector tradingHours = options["trading_hours"];
+      if( tradingHours.size() != 2 ) { throw std::invalid_argument( "trading_hours must have two elements" ); }
+      SetTradingHours( tradingHours[0], tradingHours[1] );
+
+    }
+
+    if( hasCost ) {
+
+      Rcpp::List cost = options["cost"];
+      SetCost( cost );
+
+    }
+    if( hasStop           ) SetStop            ( options["stop"           ] );
+    if( hasTradeStart     ) SetStartTradingTime( options["trade_start"    ] );
+    if( hasLatency        ) SetLatency         ( options["latency"        ] );
+    if( hasLatencyReceive ) SetLatencyReceive  ( options["latency_receive"] );
+    if( hasLatencySend    ) SetLatencySend     ( options["latency_send"   ] );
+
+  }
+
+
   void Feed( const Tick& tick ) {
 
     if( tick.time < prevTickTime ) { throw std::invalid_argument( "ticks must be time ordered tick.id = " + std::to_string( tick.id + 1 ) ); }
 
+    if( statistics.drawDown < stopTradingDrawdown ) StopTrading();
+    if( statistics.marketValue < stopTradingLoss )  StopTrading();
+
+    if( onMarketOpen  != nullptr and alarmMarketOpen .IsRinging( tick.time ) ) onMarketOpen();
+    if( onMarketClose != nullptr and alarmMarketClose.IsRinging( tick.time ) ) onMarketClose();
+
     FormCandle( tick );
 
     if( onTick != nullptr ) onTick( tick );
@@ -166,7 +260,9 @@ public:
         Trade* trade = trades[ order->idTrade ];
         if( order->IsExecuted() ) {
 
-          if( trade->IsOpened() ) {
+          trade->cost += cost.stockAbs + cost.tradeAbs + cost.tradeRel * order->priceExecuted * cost.pointValue;
+
+          if( trade->IsOpened() or trade->IsClosing() ) {
 
             trade->idExit    = order->idProcessed;
             trade->timeExit  = order->timeProcessed;
@@ -189,8 +285,6 @@ public:
 
           }
 
-          trade->cost += cost.stockAbs + cost.tradeAbs + cost.tradeRel * order->priceExecuted * cost.pointValue;
-
         }
 
         if( order->IsNew()       ) { trade->cost += cost.order;  }
@@ -239,6 +333,17 @@ public:
 
         }
 
+        if( isTradingStopped and not trade->IsClosing() ) {
+
+          Order* order = new Order( trade->IsLong() ? OrderSide::SELL : OrderSide::BUY, OrderType::MARKET, NA_REAL, "stop", trade->idTrade );
+
+          orders.push_back( order );
+          statistics.Update( order );
+
+          trade->state = TradeState::CLOSING;
+
+        }
+
       }
 
       if( trade->IsClosed() ) {
@@ -258,6 +363,12 @@ public:
 
   Statistics GetStatistics() { return statistics; }
 
+  void StopTrading() {
+
+    isTradingStopped = true;
+
+  }
+
   void Feed( Rcpp::NumericVector times, Rcpp::NumericVector prices, Rcpp::IntegerVector volumes ) {
 
     if( times.size() != prices.size() or times.size() != volumes.size() or prices.size() != volumes.size() ) {
@@ -327,6 +438,13 @@ public:
 
   void SendOrder( Order* order ) {
 
+    if( not CanTrade() ) {
+
+      delete order;
+      return;
+
+    }
+
     orders.push_back( order );
     statistics.Update( order );
 
@@ -334,6 +452,8 @@ public:
 
   void CancelOrders() { for( auto order: orders ) order->Cancel( ); }
 
+  bool CanTrade() { return not( prevTickTime < startTradingTime or isTradingStopped ); }
+
   int GetPosition() { return statistics.position; }
 
   int GetPositionPlanned() { return statistics.positionPlanned; }
@@ -356,6 +476,7 @@ public:
 
     statistics.Reset();
     prevTickTime = 0;
+    isTradingStopped = false;
   }
 
   std::vector<double> GetOnCandleMarketValueHistory() {  return statistics.onCandleHistoryMarketValue;  }
@@ -364,12 +485,13 @@ public:
 
   Rcpp::List GetOnDayClosePerformanceHistory() {
 
-    Rcpp::DataFrame performance = ListBuilder()
-
-    .Add( "date"       , IntToDate( statistics.onDayCloseHistoryDates ) )
-    .Add( "return"     , statistics.onDayCloseHistoryMarketValueChange  )
-    .Add( "pnl"        , statistics.onDayCloseHistoryMarketValue        )
-    .Add( "drawdown"   , statistics.onDayCloseHistoryDrawDown           );
+    Rcpp::List performance = ListBuilder().AsDataTable()
+      .Add( "date"     , IntToDate( statistics.onDayCloseHistoryDates ) )
+      .Add( "return"   , statistics.onDayCloseHistoryMarketValueChange  )
+      .Add( "pnl"      , statistics.onDayCloseHistoryMarketValue        )
+      .Add( "drawdown" , statistics.onDayCloseHistoryDrawDown           )
+      .Add( "avg_pnl"  , statistics.onDayCloseHistoryAvgTradePnl        )
+      .Add( "n_per_day", statistics.onDayCloseHistoryNTrades            );
 
     return performance;
 
@@ -404,7 +526,7 @@ public:
 
     for( auto it = candles.begin(); it != candles.end(); it++ ) convertCandle( it );
 
-    Rcpp::DataFrame candles = ListBuilder()
+    Rcpp::List candles = ListBuilder().AsDataTable()
 
       .Add( "time"  , time   )
       .Add( "open"  , open   )
@@ -414,7 +536,7 @@ public:
       .Add( "volume", volume )
       .Add( "id"    , id     );
 
-      return candles;
+    return candles;
 
   }
 
@@ -456,7 +578,7 @@ public:
     for( auto it = ordersProcessed.begin(); it != ordersProcessed.end(); it++ ) convertOrder( *it );
     for( auto it = orders         .begin(); it != orders         .end(); it++ ) convertOrder( *it );
 
-    Rcpp::DataFrame orders = ListBuilder()
+    Rcpp::List orders = ListBuilder().AsDataTable()
 
       .Add( "id_trade"      , id_trade       )
       .Add( "id_sent"       , id_sent        )
@@ -470,7 +592,7 @@ public:
       .Add( "state"         , state          )
       .Add( "comment"       , comment        );
 
-      return orders;
+    return orders;
 
   }
 
@@ -530,7 +652,7 @@ public:
     for( auto it = tradesProcessed.begin(); it != tradesProcessed.end(); it++ ) convertTrade( it->second );
     for( auto it = trades         .begin(); it != trades         .end(); it++ ) convertTrade( it->second );
 
-    Rcpp::DataFrame trades = ListBuilder()
+    Rcpp::List trades = ListBuilder().AsDataTable()
 
       .Add( "id_trade"   , id_trade    )
       .Add( "id_sent"    , id_sent     )
@@ -552,11 +674,11 @@ public:
       .Add( "cost_rel"   , cost_rel    )
       .Add( "state"      , state       );
 
-      return trades;
+    return trades;
 
   }
 
-  Rcpp::DataFrame GetSummary() { return statistics.GetSummary(); }
+  Rcpp::List GetSummary() { return statistics.GetSummary(); }
 
 };
 
